@@ -28,7 +28,6 @@
  */
 
 #include <sys/stat.h>
-#include <glade/glade.h>
 #include "pilot.h"
 #include "util.h"
 #include "gnome-pilot-ddialog.h"
@@ -38,7 +37,7 @@ static GtkObjectClass *parent_class = NULL;
 
 struct _GnomePilotDDialogPrivate 
 {
-	GladeXML *xml;
+	GtkBuilder *ui;
 
 	GPilotDevice *device;
 
@@ -48,13 +47,14 @@ struct _GnomePilotDDialogPrivate
 	GtkWidget *device_port;
 	GtkWidget *device_port_combo;
 	GtkWidget *device_port_label;
-	GtkWidget *device_speed;
+	GtkWidget *device_speed_combo;
 	GtkWidget *device_speed_label;
 	GtkWidget *device_timeout;
 	GtkWidget *device_serial;
 	GtkWidget *device_usb;
 	GtkWidget *device_irda;
 	GtkWidget *device_network;
+	GtkWidget *device_bluetooth;
 #ifdef PILOT_LINK_0_12
 	GtkWidget *libusb_label;
 	GList *libusb_list;
@@ -71,29 +71,29 @@ static void fill_widgets (GnomePilotDDialog *gpdd);
 
 static void gpdd_destroy (GtkObject *object);
 static void set_widget_visibility_by_type(GnomePilotDDialog *gpdd, int type);
-static void network_device_toggled_callback (GtkRadioButton *network_radio,
+static void network_device_toggled_callback (GtkRadioButton *btn,
     void *data);
 
-GtkType
+GType
 gnome_pilot_ddialog_get_type (void)
 {
-  static GtkType type = 0;
+  static GType type = 0;
 
   if (type == 0)
     {
-      static const GtkTypeInfo info =
+      static const GTypeInfo info =
       {
-        "GnomePilotDDialog",
-        sizeof (GnomePilotDDialog),
         sizeof (GnomePilotDDialogClass),
-        (GtkClassInitFunc) class_init,
-        (GtkObjectInitFunc) init,
-        /* reserved_1 */ NULL,
-        /* reserved_2 */ NULL,
-        (GtkClassInitFunc) NULL,
+        NULL,
+        NULL,
+        (GClassInitFunc) class_init,
+        NULL,
+        NULL,
+        sizeof (GnomePilotDDialog),
+	0,
+        (GInstanceInitFunc) init,
       };
-
-      type = gtk_type_unique (gtk_object_get_type (), &info);
+      type = g_type_register_static (gtk_object_get_type (), "GnomePilotDDialog", &info, 0);
     }
 
   return type;
@@ -106,7 +106,7 @@ class_init (GnomePilotDDialogClass *klass)
 
 	object_class = GTK_OBJECT_CLASS (klass);
 
-	parent_class = gtk_type_class (gtk_object_get_type ());
+	parent_class = g_type_class_peek (gtk_object_get_type ());
 
 	object_class->destroy = gpdd_destroy;
 }
@@ -115,20 +115,30 @@ static void
 init (GnomePilotDDialog *gpdd)
 {
 	GnomePilotDDialogPrivate *priv;
+	guint error;
+	GError *err = NULL;
+	gchar *objects[] = {"DeviceSettings",
+			    "timeout_adjustment",
+			    "device_port_store",
+			    NULL};
 	
 	priv = g_new0 (GnomePilotDDialogPrivate, 1);
 
 	gpdd->priv = priv;
 
 	/* Gui stuff */
-	priv->xml = glade_xml_new (GLADEDATADIR "/gpilotd-capplet.glade", "DeviceSettings", NULL);
-	if (!priv->xml) {
-		g_message ("gnome-pilot-ddialog init(): Could not load the Glade XML file!");
-		goto error;
+	priv->ui = gtk_builder_new ();
+	error = gtk_builder_add_objects_from_file (priv->ui, "gpilotd-capplet.ui", objects, NULL);
+	if (error == 0) {
+       	error = gtk_builder_add_objects_from_file (priv->ui, UIDATADIR "/gpilotd-capplet.ui", objects, &err);
+		if (error == 0) {
+			g_message ("gnome-pilot-ddialog init(): Could not load the GtkBuilder UI file: %s", err->message);
+			goto error;
+		}
 	}
 
 	if (!get_widgets (gpdd)) {
-		g_message ("gnome-pilot-ddialog init(): Could not find all widgets in the XML file!");
+		g_message ("gnome-pilot-ddialog init(): Could not find all widgets in the UI file!");
 		goto error;
 	}
 
@@ -144,7 +154,7 @@ gnome_pilot_ddialog_new (GPilotDevice *device)
 	GnomePilotDDialog *gpdd;
 	GtkObject *object;
 	
-	object = gtk_type_new (GNOME_PILOT_TYPE_DDIALOG);
+	object = GTK_OBJECT (g_type_create_instance (GNOME_PILOT_TYPE_DDIALOG));
 	
 	gpdd = GNOME_PILOT_DDIALOG (object);
 	gpdd->priv->device = device;
@@ -160,46 +170,63 @@ static gboolean
 get_widgets (GnomePilotDDialog *gpdd)
 {
 	GnomePilotDDialogPrivate *priv;
+	GtkTreeModel *model;
+	GtkTreeIter iter;
 
 	priv = gpdd->priv;
 
-#define GW(name) glade_xml_get_widget (priv->xml, name)
+#define GW(name) GTK_WIDGET (gtk_builder_get_object (priv->ui, name))
 
 	priv->dialog = GW ("DeviceSettings");
 
 	priv->device_name = GW ("device_name_entry");
-	priv->device_port = GW ("device_port_entry");
 	priv->device_port_label = GW ("device_port_label");
 	priv->device_port_combo = GW ("device_port_combo");
-	priv->device_speed = GW ("device_speed_menu");
+	priv->device_port = gtk_bin_get_child (
+	    GTK_BIN(priv->device_port_combo));
+	priv->device_speed_combo = GW ("device_speed_combo");
 	priv->device_speed_label = GW ("device_speed_label");
 	priv->device_timeout = GW ("timeout_spinner");
 	priv->device_serial = GW ("serial_radio");
 	priv->device_usb = GW ("usb_radio");
 	priv->device_irda = GW ("irda_radio");
 	priv->device_network = GW ("network_radio");
+	priv->device_bluetooth = GW ("bluetooth_radio");
 
+	/* Doing the cell layout in glade .ui file seemed to result
+	 * in duplicated text.  Probably a bug in glade/gtkbuilder?
+	 */
+	//	model = GTK_TREE_MODEL(gtk_list_store_new (1, G_TYPE_STRING));
+	model = GTK_TREE_MODEL(gtk_builder_get_object (priv->ui,
+		"device_port_store"));
+	gtk_combo_box_set_model (GTK_COMBO_BOX (priv->device_port_combo), model);
+	gtk_combo_box_entry_set_text_column(
+	    GTK_COMBO_BOX_ENTRY (priv->device_port_combo), 0);
+	gtk_cell_layout_clear(GTK_CELL_LAYOUT(priv->device_port_combo));
+	GtkCellRenderer *renderer = gtk_cell_renderer_text_new();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (priv->device_port_combo),
+	    renderer, TRUE);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (priv->device_port_combo),
+	    renderer, "text", 0);
 #ifdef PILOT_LINK_0_12
 	/* usb: (libusb) pseudo-device is available from pilot-link 0.12.0 */
-	priv->libusb_list = NULL;
-	priv->libusb_label = gtk_list_item_new_with_label ("usb:");
-	gtk_widget_show(priv->libusb_label);
-	priv->libusb_list = g_list_append(priv->libusb_list,
-	    priv->libusb_label);
-	gtk_list_insert_items (GTK_LIST((GTK_COMBO(priv->device_port_combo))->list),
-	    priv->libusb_list, 1);
+	gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
+	gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, "usb:", -1);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(priv->device_usb), 1);
 #endif
-	
+	fill_speed_combo(GTK_COMBO_BOX(priv->device_speed_combo));
+
 #undef GW
 	return (priv->dialog
 		&& priv->device_name
 		&& priv->device_port
-		&& priv->device_speed
+		&& priv->device_speed_combo
 		&& priv->device_timeout
 		&& priv->device_serial
 		&& priv->device_usb
 		&& priv->device_irda
-		&& priv->device_network);
+		&& priv->device_network
+		&& priv->device_bluetooth);
 }
 
 static void
@@ -209,14 +236,15 @@ map_widgets (GnomePilotDDialog *gpdd)
 	
 	priv = gpdd->priv;
 
-	gtk_object_set_data (GTK_OBJECT (gpdd), "port_entry", priv->device_port);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "name_entry", priv->device_name);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "speed_menu", priv->device_speed);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "irda_radio", priv->device_serial);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "usb_radio", priv->device_usb);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "irda_radio", priv->device_irda);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "network_radio", priv->device_network);
-	gtk_object_set_data (GTK_OBJECT (gpdd), "timeout_spinner", priv->device_timeout);
+	g_object_set_data (G_OBJECT (gpdd), "port_entry", priv->device_port);
+	g_object_set_data (G_OBJECT (gpdd), "name_entry", priv->device_name);
+	g_object_set_data (G_OBJECT (gpdd), "speed_combo", priv->device_speed_combo);
+	g_object_set_data (G_OBJECT (gpdd), "irda_radio", priv->device_serial);
+	g_object_set_data (G_OBJECT (gpdd), "usb_radio", priv->device_usb);
+	g_object_set_data (G_OBJECT (gpdd), "irda_radio", priv->device_irda);
+	g_object_set_data (G_OBJECT (gpdd), "network_radio", priv->device_network);
+	g_object_set_data (G_OBJECT (gpdd), "bluetooth_radio", priv->device_bluetooth);
+	g_object_set_data (G_OBJECT (gpdd), "timeout_spinner", priv->device_timeout);
 }
 
 static void 
@@ -226,10 +254,12 @@ init_widgets (GnomePilotDDialog *gpdd)
 
 	priv = gpdd->priv;
 
-	gtk_signal_connect (GTK_OBJECT (priv->device_port),"insert-text",
-			    GTK_SIGNAL_FUNC (insert_device_callback), NULL);
-	gtk_signal_connect (GTK_OBJECT (priv->device_network), "toggled",
-			    GTK_SIGNAL_FUNC (network_device_toggled_callback), gpdd);
+	g_signal_connect (GTK_OBJECT (priv->device_port),"insert-text",
+			    G_CALLBACK (insert_device_callback), NULL);
+	g_signal_connect (GTK_OBJECT (priv->device_bluetooth), "toggled",
+			    G_CALLBACK (network_device_toggled_callback), gpdd);
+	g_signal_connect (GTK_OBJECT (priv->device_network), "toggled",
+			    G_CALLBACK (network_device_toggled_callback), gpdd);
 }
 
 static void
@@ -241,9 +271,8 @@ fill_widgets (GnomePilotDDialog *gpdd)
 
 	if (priv->device) {
 		gtk_entry_set_text (GTK_ENTRY (priv->device_name), priv->device->name);
-		gtk_entry_set_text (GTK_ENTRY (priv->device_port), priv->device->port);
-
-		fill_speed_menu (GTK_OPTION_MENU (priv->device_speed), priv->device->speed);
+		if (priv->device->port != NULL)
+			gtk_entry_set_text (GTK_ENTRY (priv->device_port), priv->device->port);
 
 		gtk_spin_button_set_value (GTK_SPIN_BUTTON (priv->device_timeout), priv->device->timeout);
 
@@ -255,7 +284,11 @@ fill_widgets (GnomePilotDDialog *gpdd)
 					      priv->device->type == PILOT_DEVICE_IRDA);
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->device_network),
 					      priv->device->type == PILOT_DEVICE_NETWORK);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->device_bluetooth),
+					      priv->device->type == PILOT_DEVICE_BLUETOOTH);
 		set_widget_visibility_by_type(gpdd, priv->device->type);
+		speed_combo_set_speed(GTK_COMBO_BOX(priv->device_speed_combo),
+				      priv->device->speed);
 	}
 }
 
@@ -265,6 +298,7 @@ gnome_pilot_ddialog_run_and_close (GnomePilotDDialog *gpdd, GtkWindow *parent)
 	GnomePilotDDialogPrivate *priv;
 	gint btn;
 	GPilotDevice *tmpdev = g_new0 (GPilotDevice, 1);
+	gchar *errstr = NULL;
 	
 	priv = gpdd->priv;
 
@@ -274,9 +308,12 @@ gnome_pilot_ddialog_run_and_close (GnomePilotDDialog *gpdd, GtkWindow *parent)
 	
 		if (btn == GTK_RESPONSE_OK) {
 			read_device_config (GTK_OBJECT (gpdd), tmpdev);
-			if(check_device_settings (tmpdev)) {
+			if(check_device_settings (tmpdev, &errstr)) {
 				*priv->device = *tmpdev;
 				break;
+			} else {
+				error_dialog(NULL, errstr);
+				g_free(errstr);
 			}
 		} else {
 			break;
@@ -297,14 +334,14 @@ gpdd_destroy (GtkObject *object)
 	priv = gpdd->priv;
 
 	gtk_widget_destroy (priv->dialog);
-	gtk_object_unref (GTK_OBJECT (priv->xml));
+	g_object_unref (G_OBJECT (priv->ui));
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
 		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
 }
 
 static void
-network_device_toggled_callback (GtkRadioButton *network_radio, void *data)
+network_device_toggled_callback (GtkRadioButton *btn, void *data)
 {
 	GnomePilotDDialog *gpdd = (GnomePilotDDialog *)data;
 	GnomePilotDDialogPrivate *priv;
@@ -312,8 +349,13 @@ network_device_toggled_callback (GtkRadioButton *network_radio, void *data)
 
 	priv = gpdd->priv;
 
-	if(GTK_TOGGLE_BUTTON(network_radio)->active) {
+	/* toggled button could be bluetooth or network */
+	if(btn == GTK_RADIO_BUTTON(priv->device_network) &&
+	    GTK_TOGGLE_BUTTON(btn)->active) {
 		type = PILOT_DEVICE_NETWORK;
+	} else if (btn == GTK_RADIO_BUTTON(priv->device_bluetooth) &&
+	    GTK_TOGGLE_BUTTON(btn)->active) {
+		type = PILOT_DEVICE_BLUETOOTH;
 	} else {
 		type = PILOT_DEVICE_SERIAL;
 	}
@@ -323,15 +365,17 @@ network_device_toggled_callback (GtkRadioButton *network_radio, void *data)
 static void
 set_widget_visibility_by_type(GnomePilotDDialog *gpdd, int type) {
 	GnomePilotDDialogPrivate *priv;
+	gboolean enable_extra_widgets = (type != PILOT_DEVICE_NETWORK &&
+	    type != PILOT_DEVICE_BLUETOOTH);
 
 	priv = gpdd->priv;
 
 	gtk_widget_set_sensitive(priv->device_port_combo,
-	    type != PILOT_DEVICE_NETWORK);
+	    enable_extra_widgets);
 	gtk_widget_set_sensitive(priv->device_port_label,
-	    type != PILOT_DEVICE_NETWORK);
-	gtk_widget_set_sensitive(priv->device_speed,
-	    type != PILOT_DEVICE_NETWORK);
+	    enable_extra_widgets);
+	gtk_widget_set_sensitive(priv->device_speed_combo,
+	    enable_extra_widgets);
 	gtk_widget_set_sensitive(priv->device_speed_label,
-	    type != PILOT_DEVICE_NETWORK);
+	    enable_extra_widgets);
 }
