@@ -233,7 +233,6 @@ loadPilotPilot (PilotState *p)
 {
   GPilotPilot *pilot;
   gchar *iPilot;
-  char *local_name;
   int i, num;
   GKeyFile *kfile;
   GError *error = NULL;
@@ -270,15 +269,8 @@ loadPilotPilot (PilotState *p)
 			  pilot->pilot_id = getuid ();
                   }
 		  g_message ("Pilot id   -> %d", pilot->pilot_id);
-		  local_name = g_key_file_get_string (kfile, iPilot, "pilotusername", NULL);
-#ifdef PILOT_LINK_0_12
-		  if (!local_name || (convert_FromPilotChar_WithCharset ("UTF-8", local_name, strlen(local_name), &pilot->pilot_username, NULL) == -1)) {
-#else
-		  if (!local_name || (convert_FromPilotChar ("UTF-8", local_name, strlen(local_name), &pilot->pilot_username) == -1)) {
-#endif
-  		  	pilot->pilot_username = g_strdup (local_name);
-		  }
-		  g_free (local_name);
+		  /* username is stored as utf8 */
+		  pilot->pilot_username = g_key_file_get_string (kfile, iPilot, "pilotusername", NULL);
 		  g_message ("Pilot username -> %s", pilot->pilot_username);
 	  
 		  pilot->creation = g_key_file_get_integer (kfile, iPilot, "creation", NULL);
@@ -345,7 +337,6 @@ savePilotState (PilotState *state)
   GList *tmp;
   gchar *iDevice;
   gchar *iPilot;
-  char *local_name;
   GKeyFile *kfile;
 
   kfile = get_gpilotd_kfile ();
@@ -394,15 +385,8 @@ savePilotState (PilotState *state)
 	  g_key_file_set_integer (kfile, iPilot, "pilotid", pilot->pilot_id);
 	  g_key_file_set_integer (kfile, iPilot, "creation", pilot->creation);
 	  g_key_file_set_integer (kfile, iPilot, "romversion", pilot->romversion);
-#ifdef PILOT_LINK_0_12
-	  if (!pilot->pilot_username|| (convert_ToPilotChar_WithCharset ("UTF-8", pilot->pilot_username, strlen (pilot->pilot_username), &local_name, NULL) == -1)) {
-#else
-	  if (!pilot->pilot_username|| (convert_ToPilotChar ("UTF-8", pilot->pilot_username, strlen (pilot->pilot_username), &local_name) == -1)) {
-#endif
-		  local_name = g_strdup (pilot->pilot_username);
-	  }
-	  g_key_file_set_string (kfile, iPilot, "pilotusername", local_name);
-	  g_free (local_name);
+	  /* store username as utf8 */
+	  g_key_file_set_string (kfile, iPilot, "pilotusername", pilot->pilot_username);
 	  g_key_file_set_string (kfile, iPilot, "basedir", pilot->sync_options.basedir);
 #ifdef PILOT_LINK_0_12
 	  g_key_file_set_string (kfile, iPilot, "charset", pilot->pilot_charset);
