@@ -21,14 +21,15 @@
  *          Robert Mibus
  */
 
-/* $Id: backup_conduit.c 1524 2008-01-03 09:12:45Z mcdavey $ */
+/* $Id$ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
+#include <stdio.h>
 #include <glib.h>
-#include <gnome.h>
+#include <glib/gi18n.h>
 
 #include <pi-source.h>
 #include <pi-socket.h>
@@ -49,6 +50,7 @@
 #include <dirent.h>
 
 #include <gpilotd/gnome-pilot-conduit-backup.h>
+#include <gpilotd/gnome-pilot-config.h>
 #include "backup_conduit.h"
 
 #define DEBUG 1
@@ -88,28 +90,43 @@ load_configuration(GnomePilotConduit *conduit,
 		   GPilotPilot *pilot)
 {
 	gchar *temp_name;
-	gchar *prefix;
+	gchar *iPilot;
 	gchar **exclude_files;
-	gint num_of_exclude_files = 0;
+	gsize num_of_exclude_files = 0;
 	guint i;
 	DIR *dir;
 	struct dirent *entry;
+  	GKeyFile *kfile;
+	GError   *error = NULL;
 
 	(*c) = g_new0(ConduitCfg,1);
 	(*c)->child = -1;
 	
-	prefix = g_strdup_printf(CONFIG_PREFIX, pilot->pilot_id);
-	
-	gnome_config_push_prefix(prefix);
-	(*c)->backup_dir = gnome_config_get_string("backup_dir");
-	(*c)->updated_only = gnome_config_get_bool("updated_only=TRUE");
-	(*c)->remove_deleted = gnome_config_get_bool("remove_deleted=FALSE");	
-	(*c)->no_of_backups = gnome_config_get_int("no_of_backups=0");
+ 	kfile = get_backup_kfile ();
+	iPilot = g_strdup_printf ("Pilot_%u", pilot->pilot_id);
+	(*c)->backup_dir = g_key_file_get_string (kfile, iPilot, "backup_dir", NULL);
+
+	(*c)->updated_only = g_key_file_get_boolean (kfile, iPilot, "updated_only", &error);
+	if (error) {
+		g_warning (_("Unable load key backup-conduit/%s/updated_only: %s"), iPilot, error->message);
+		g_error_free (error);
+		error = NULL;
+		(*c)->updated_only = TRUE;
+	}
+
+	(*c)->remove_deleted = g_key_file_get_boolean (kfile, iPilot, "remove_deleted", NULL);	
+	if (error) {
+		g_warning (_("Unable load key backup-conduit/%s/remove_deleted: %s"), iPilot, error->message);
+		g_error_free (error);
+		error = NULL;
+		(*c)->remove_deleted = FALSE;
+	}
+
+	(*c)->no_of_backups = g_key_file_get_integer (kfile, iPilot, "no_of_backups", NULL);
 
 	(*c)->exclude_files = NULL;
-	gnome_config_get_vector("exclude_files",
-				&num_of_exclude_files,
-				&exclude_files);
+	exclude_files = g_key_file_get_string_list (kfile, iPilot, "exclude_files",
+						    &num_of_exclude_files, NULL);
 	if(num_of_exclude_files) {
 		for( i = 0; i < num_of_exclude_files ; i++ ) {
 			(*c)->exclude_files = g_list_append( (*c)->exclude_files , 
@@ -118,7 +135,6 @@ load_configuration(GnomePilotConduit *conduit,
 		}
 		g_free(exclude_files);
 	}
-	gnome_config_pop_prefix();
 	
 	if ((*c)->backup_dir == NULL) {
 		if (conduit != NULL && GNOME_IS_PILOT_CONDUIT (conduit)) {
@@ -167,20 +183,23 @@ load_configuration(GnomePilotConduit *conduit,
 
 	(*c)->pilotId = pilot->pilot_id;
 
-	g_free(prefix);
+	g_free (iPilot);
+	g_key_file_free (kfile);
 }
 
 static void 
 save_configuration(ConduitCfg *c) 
 {
-	gchar *prefix;
-	const gchar **exclude = NULL;
+	gchar *iPilot;
+	const gchar **exclude;
 	GList *iterator;
 	guint i = 0;
+	GKeyFile *kfile;
 
 	g_return_if_fail(c!=NULL);
        	
-	prefix= g_strdup_printf("/gnome-pilot.d/backup-conduit/Pilot_%u/",c->pilotId);
+	kfile = get_backup_kfile ();
+	iPilot = g_strdup_printf("Pilot_%u",c->pilotId);
 	
 	if( c->exclude_files != NULL ) {
 	  iterator = c->exclude_files;
@@ -190,17 +209,21 @@ save_configuration(ConduitCfg *c)
 	  }
 	  exclude[i] = NULL;
 	}
-	gnome_config_push_prefix(prefix);
-	gnome_config_set_string("backup_dir",c->backup_dir);
-	gnome_config_set_bool("updated_only",c->updated_only);
-	gnome_config_set_bool("remove_deleted",c->remove_deleted);
-	gnome_config_set_int("no_of_backups",c->no_of_backups);
-	gnome_config_set_vector("exclude_files", i, exclude);
-	gnome_config_pop_prefix();
-	gnome_config_sync();
-	gnome_config_drop_all();
-	g_free(prefix);
-	g_free(exclude);
+	g_key_file_set_string (kfile, iPilot, "backup_dir",c->backup_dir);
+	g_key_file_set_boolean (kfile, iPilot, "updated_only",c->updated_only);
+	g_key_file_set_boolean (kfile, iPilot, "remove_deleted",c->remove_deleted);
+	g_key_file_set_integer (kfile, iPilot, "no_of_backups",c->no_of_backups);
+	if (i != 0) {
+		g_key_file_set_string_list (kfile, iPilot, "exclude_files", exclude, i);
+		g_free(exclude);
+	} else {
+		g_key_file_set_string (kfile, iPilot, "exclude_files", "");
+	}
+
+	g_free(iPilot);
+
+	save_backup_kfile (kfile);
+	g_key_file_free (kfile);
 }
 
 static void 
@@ -244,13 +267,15 @@ destroy_configuration(ConduitCfg **c)
 		for (iterator = (*c)->files_in_backup; iterator; iterator = g_list_next (iterator)) {
 			gchar *filename = (char*)iterator->data;
 			gchar *backup_dirname;
+			gchar *basename;
 			gchar *backup_backup;
 			
 			/* I18N note: this is printed when renaming a file %s */
 			g_message (_("Renaming %s"), filename);
 
-			backup_dirname = g_dirname (filename);
-			backup_backup = g_strdup_printf ("%s/del/%s", backup_dirname, g_basename (filename));
+			backup_dirname = g_path_get_dirname (filename);
+			basename = g_path_get_basename (filename);
+			backup_backup = g_strdup_printf ("%s/del/%s", backup_dirname, basename);
 
 			/* I18N note: this message follow a "Renaming %s". The
 			   %s is a file name */
@@ -260,6 +285,7 @@ destroy_configuration(ConduitCfg **c)
 			}
 
 			g_free (backup_backup);
+			g_free (basename);
 			g_free (backup_dirname);
 			g_free (filename);
 		}
@@ -306,7 +332,7 @@ gnome_pilot_conduit_backup_remove_deleted (GnomePilotConduitBackup *conduit,
 		GList *iterator;
 		for (iterator = cfg->files_in_backup; iterator; iterator = g_list_next (iterator)) {
 			char *filename = (char*)iterator->data;
- 			if (filename && strcasecmp(filename, name) == 0) {
+			if (filename && g_ascii_strcasecmp (filename, name) == 0) {
 				cfg->files_in_backup = g_list_remove_link (cfg->files_in_backup, 
 									   iterator);
 				g_free (filename);
@@ -410,7 +436,7 @@ gnome_real_pilot_conduit_backup_backup (GnomePilotConduitBackup *conduit,
 	
 	/* let's see first if we should actually touch this db */
 	for (iterator = cfg->exclude_files; iterator; iterator = g_list_next (iterator)) {
-		if( ! g_strcasecmp(iterator->data, PI_DBINFO (dbinfo)->name) ) {
+		if(!g_ascii_strcasecmp (iterator->data, PI_DBINFO (dbinfo)->name)) {
 			g_message("excluded %s",PI_DBINFO (dbinfo)->name);
 			result = 2;
 			goto exit;
@@ -711,10 +737,12 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 					 gpointer _cfg)
 {
 	DIR *dir;
+	struct stat buf;
 	struct dirent *dirent;
 	file_db **db;
 	int dbcount = 0;
-	int i, j, size;
+	int i, j;
+	size_t size;
 	struct pi_file *file;
 	char *source;
 	int result = 0;
@@ -742,8 +770,11 @@ gnome_real_pilot_conduit_backup_restore (GnomePilotConduitBackup *conduit,
 		printf ("checking %s/%s\n", source, dirent->d_name);
 #endif
 
-		/* skip dotfiles. */
-		if (dirent->d_name[0] == '.')
+		gchar *entryname = g_strdup_printf("%s/%s", source, dirent->d_name);
+		/* skip dotfiles and directories. */
+		lstat (entryname, &buf);
+		g_free(entryname);
+		if (dirent->d_name[0] == '.' || S_ISDIR (buf.st_mode))
 			continue;
 
 		db[dbcount] = g_new0 (file_db, 1);
@@ -827,13 +858,13 @@ insert_dir_callback (GtkEditable *editable, const gchar *text,
 	curname = gtk_entry_get_text(GTK_ENTRY(editable));
 	if (*curname == '\0' && len > 0) {
 		if (isspace(text[0])) {
-			gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), "insert_text");
+			g_signal_stop_emission_by_name (GTK_OBJECT(editable), "insert_text");
 			return;
 		}
 	} else {
 		for (i=0; i<len; i++) {
 			if (isspace(text[i])) {
-				gtk_signal_emit_stop_by_name(GTK_OBJECT(editable), 
+				g_signal_stop_emission_by_name (GTK_OBJECT(editable), 
 							     "insert_text");
 				return;
 			}
@@ -850,22 +881,23 @@ static GtkWidget
 	GtkObject *adjustment;
 	AtkObject *atk_widget;
 
-	vbox = gtk_vbox_new(FALSE, GNOME_PAD);
+	vbox = gtk_vbox_new (FALSE, 8);
 
 	table = gtk_table_new(2, 4, FALSE);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 4);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 10);
-	gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, GNOME_PAD);
+	gtk_box_pack_start (GTK_BOX(vbox), table, FALSE, FALSE, 8);
 
 	label = gtk_label_new_with_mnemonic(_("_Backup directory:"));
 	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
     
-	entry = gtk_entry_new_with_max_length(128);
-	gtk_object_set_data(GTK_OBJECT(vbox), "dir", entry);
+	entry = gtk_entry_new ();
+	gtk_entry_set_max_length(GTK_ENTRY(entry), 128);
+	g_object_set_data (G_OBJECT(vbox), "dir", entry);
 	gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
-	gtk_signal_connect(GTK_OBJECT(entry), "insert_text",
-			   GTK_SIGNAL_FUNC(insert_dir_callback),
+	g_signal_connect(GTK_OBJECT(entry), "insert_text",
+			   G_CALLBACK(insert_dir_callback),
 			   NULL);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), entry);
@@ -877,7 +909,7 @@ static GtkWidget
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 1,2);
 
 	button = gtk_check_button_new();
-	gtk_object_set_data(GTK_OBJECT(vbox), "only_changed", button);
+	g_object_set_data (G_OBJECT(vbox), "only_changed", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 1,2);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
@@ -889,7 +921,7 @@ static GtkWidget
 	gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 2,3);
 
 	button = gtk_check_button_new();
-	gtk_object_set_data(GTK_OBJECT(vbox), "remove_local", button);
+	g_object_set_data (G_OBJECT(vbox), "remove_local", button);
 	gtk_table_attach_defaults(GTK_TABLE(table), button, 1, 2, 2,3);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), button);
@@ -902,7 +934,7 @@ static GtkWidget
 
 	adjustment = gtk_adjustment_new (0, 0, 100, 1, 1, 1);
 	spin = gtk_spin_button_new (GTK_ADJUSTMENT(adjustment), 1, 0);
-	gtk_object_set_data(GTK_OBJECT(vbox), "no_of_backups", adjustment);
+	g_object_set_data (G_OBJECT(vbox), "no_of_backups", adjustment);
 	gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 3, 4);
 
 	gtk_label_set_mnemonic_widget (GTK_LABEL(label), spin);
@@ -923,7 +955,7 @@ error_dialog (GtkWindow *parent, gchar *mesg, ...)
         tmp = g_strdup_vprintf (mesg,ap);
 
         dlg = gtk_message_dialog_new (parent, GTK_DIALOG_DESTROY_WITH_PARENT,
-                                      GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, tmp);
+	    GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, "%s", tmp);
         gtk_dialog_run (GTK_DIALOG (dlg));
         gtk_widget_destroy (dlg);
 
@@ -981,10 +1013,10 @@ setOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 	GtkWidget *dir,*updated_only,*remove_deleted;
 	GtkObject *adj;
 
-	dir  = gtk_object_get_data(GTK_OBJECT(pilotcfg), "dir");
-	updated_only = gtk_object_get_data(GTK_OBJECT(pilotcfg), "only_changed");
-	remove_deleted = gtk_object_get_data(GTK_OBJECT(pilotcfg), "remove_local");
-	adj = gtk_object_get_data(GTK_OBJECT(pilotcfg), "no_of_backups");
+	dir  = g_object_get_data (G_OBJECT(pilotcfg), "dir");
+	updated_only = g_object_get_data (G_OBJECT(pilotcfg), "only_changed");
+	remove_deleted = g_object_get_data (G_OBJECT(pilotcfg), "remove_local");
+	adj = g_object_get_data (G_OBJECT(pilotcfg), "no_of_backups");
 
 	g_assert(dir!=NULL);
 	g_assert(updated_only!=NULL);
@@ -1005,10 +1037,10 @@ readOptionsCfg(GtkWidget *pilotcfg, ConduitCfg *state)
 	GtkWidget *dir,*updated_only,*remove_deleted;
 	GtkObject *adj;
 
-	dir  = gtk_object_get_data(GTK_OBJECT(pilotcfg), "dir");
-	updated_only = gtk_object_get_data(GTK_OBJECT(pilotcfg), "only_changed");
-	remove_deleted = gtk_object_get_data(GTK_OBJECT(pilotcfg), "remove_local");
-	adj = gtk_object_get_data(GTK_OBJECT(pilotcfg), "no_of_backups");
+	dir  = g_object_get_data (G_OBJECT(pilotcfg), "dir");
+	updated_only = g_object_get_data (G_OBJECT(pilotcfg), "only_changed");
+	remove_deleted = g_object_get_data (G_OBJECT(pilotcfg), "remove_local");
+	adj = g_object_get_data (G_OBJECT(pilotcfg), "no_of_backups");
 
 	if(state->backup_dir)
 		g_free(state->backup_dir);	
@@ -1032,7 +1064,7 @@ create_settings_window (GnomePilotConduit *conduit, GtkWidget *parent, gpointer
 	gtk_container_add(GTK_CONTAINER(parent),cfgWindow);
 	gtk_widget_show_all(cfgWindow);
 
-	gtk_object_set_data(GTK_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
+	g_object_set_data (G_OBJECT(conduit),OBJ_DATA_CONFIG_WINDOW,cfgWindow);
 	setOptionsCfg(GET_CONDUIT_WINDOW(conduit),GET_CONDUIT_CFG(conduit));
 
 	return 0;
@@ -1076,10 +1108,10 @@ conduit_load_gpilot_conduit (GPilotPilot *pilot)
 
 	cfg2 = dupe_configuration(cfg);
 
-	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_CONFIG,cfg);
-	gtk_object_set_data(GTK_OBJECT(retval),OBJ_DATA_OLDCONFIG,cfg2);
+	g_object_set_data (G_OBJECT(retval),OBJ_DATA_CONFIG,cfg);
+	g_object_set_data (G_OBJECT(retval),OBJ_DATA_OLDCONFIG,cfg2);
 
-	gtk_object_set_data(GTK_OBJECT(retval),
+	g_object_set_data (G_OBJECT(retval),
 			    "configuration",
 			    cfg);
 
